Write-Host
Write-Host("=======================================================")
Write-Host("Web statistics recalculation tool for Plesk 17")
Write-Host("wsrecalc_12_0.ps1 <domain> <from date (dd.MM.yy)>")
Write-Host("=======================================================")
Write-Host

$ErrorActionPreference = "Stop"

if ( $args.length -eq 0 ) {
    $args += Read-Host "Enter a domain name"
}
if ( $args.length -eq 1 ) {
    $args += Read-Host "Enter date from (dd.MM.yy)"
    Write-Host
}

Write-Host("DOMAIN NAME:")
$domain_name = $args[0]
Write-Host($domain_name)
Write-Host

Write-Host("DOMAIN ID:")
$domain_name_quoted = "'" + $domain_name + "'"
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT id FROM domains WHERE name=$domain_name_quoted`"`"'
$cmd_out = Invoke-Expression $cmd
$domain_id = [int](($cmd_out)[2])
if ( $LastExitCode -ne 0 -or $domain_id -le 0 ) {
    Write-Host("ERROR: Cannot determinate domain ID. Output:")
    Write-Host($cmd_out)
    Exit(1);
}
Write-Host($domain_id)
Write-Host

Write-Host("DOMAIN VENDOR ID:")
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT vendor_id FROM domains WHERE id=$domain_id`"`"'
$cmd_out = Invoke-Expression $cmd
$domain_vendor_id = [int](($cmd_out)[2].Trim())
if ( $LastExitCode -ne 0 -or $domain_vendor_id -le 0 ) {
    Write-Host("ERROR: Cannot determinate domain vendor ID. Output:")
    Write-Host($cmd_out)
    Exit(2);
}
Write-Host($domain_vendor_id)
Write-Host

Write-Host("DOMAIN VENDOR NAME:")
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT pname FROM clients WHERE id=$domain_vendor_id`"`"'
$cmd_out = Invoke-Expression $cmd
$domain_vendor_name = ($cmd_out)[2].Trim()
if ( $LastExitCode -ne 0 -or $domain_vendor_name.length -eq 0 ) {
    Write-Host("ERROR: Cannot determinate domain vendor name. Output:")
    Write-Host($cmd_out)
    Exit(3);
}
Write-Host($domain_vendor_name)
Write-Host

Write-Host("DOMAIN VENDOR TYPE:")
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT type FROM clients WHERE id=$domain_vendor_id`"`"'
$cmd_out = Invoke-Expression $cmd
$domain_vendor_type = ($cmd_out)[2].Trim()
if ( $LastExitCode -ne 0 -or $domain_vendor_type.length -eq 0 ) {
    Write-Host("ERROR: Cannot determinate domain vendor type. Output:")
    Write-Host($cmd_out)
    Exit(4);
}
Write-Host($domain_vendor_type)
Write-Host


Write-Host("DOMAIN LOGS PATH:")
$cmd = 'cmd /C `"`"%plesk_bin64%\websrvmng.exe`" --get-vhost-log-dir --vhost-name=$domain_name`"'
$domain_logs_location = Invoke-Expression $cmd
if ( $LastExitCode -ne 0 -or $domain_logs_location.Trim().length -eq 0 ) {
    Write-Host("ERROR: Cannot determinate domain logs path. Output:")
    Write-Host($domain_logs_location)
    Exit(5);
}
Write-Host($domain_logs_location)
Write-Host

Write-Host("DOMAIN WEBSTAT PATH:")
$domain_webstat_path_pi = Resolve-Path($domain_logs_location + "\..\..\..\.plesk\statistics\$domain_name\webstat")
$domain_webstat_path = $domain_webstat_path_pi.ToString()
Write-Host($domain_webstat_path)
Write-Host

Write-Host("DOMAIN WEBSTAT TYPE:")
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT webstat FROM hosting h INNER JOIN domains d ON h.dom_id = d.id WHERE d.id=$domain_id`"`"'
$cmd_out = Invoke-Expression $cmd
$domain_webstat_type = ($cmd_out)[2].Trim()
if ( $LastExitCode -ne 0 -or $domain_webstat_type.length -eq 0 ) {
    Write-Host("ERROR: Cannot determinate domain webstat type. Output:")
    Write-Host($cmd_out)
    Exit(6);
}
if ( $domain_webstat_type -ne "awstats" -and $domain_webstat_type -ne "webalizer" ) {
    Write-Host("ERROR: Neither AWStats nor Webalizer are detected. Webstat type:")
    Write-Host($domain_webstat_type)
    Exit(7)
}
Write-Host($domain_webstat_type)
Write-Host

Write-Host("FROM DATE (yyyy/MM/dd HH:mm:ss):")
$from_date = [datetime]::ParseExact($args[1], "dd.MM.yy", $CultureInfo.InvariantCulture)
$from_date_formatted = $from_date.ToString("yyyy/MM/dd HH:mm:ss")
Write-Host($from_date_formatted)
Write-Host

if ( $domain_webstat_type -eq "awstats" -and $from_date.Day -ne 20 ) {
    Write-Host("WARNING: Updating of AWStats database file may not work properly.")
    Write-Host("So, it is better to use date with day=20 to remove and recreate database file entirely.")
    Write-Host
}
if ( $domain_webstat_type -eq "webalizer" ) {
    Write-Host("WARNING: Webalized database will be completely removed and recalculated.")
    Write-Host("So, it is better to use such early date as possible.")
    Write-Host
}

Write-Host("ATTANTION: Check data above.")
Write-Host("Backup logs folder, webstat folder, system database and")
Write-Host("press any key to continue (Ctrl-C to break)...")
$x = $host.UI.RawUI.ReadKey("NoEcho,IncludeKeyUp")

Write-Host
date
Write-Host("=================================================")
Write-Host

Write-Host("Unzipping logs...")
$zip_location = $domain_logs_location + "\*.log.zip"
$zip_list = dir $zip_location
foreach ($file in $zip_list) {
    if ( $file.LastWriteTime -lt $from_date ) {
        continue;
    }

    Write-Host("Unzip: " + $file)
    $filename = $domain_logs_location + "\" + $file.name.ToString()

    $shell_app=new-object -com shell.application
    $zip_file = $shell_app.namespace($filename)
    $destination = $shell_app.namespace($domain_logs_location)
    $destination.Copyhere($zip_file.items(), 16)
}
Write-Host

Write-Host("Updating registry...")
REG ADD "HKLM\SOFTWARE\PLESK\PSA Config\Config\Statistics\DomainIds\$domain_id" /v LastHttpStatisticsRun /t REG_SZ /d "$from_date_formatted" /reg:32 /f
if ( $LastExitCode -ne 0 ) {
    Write-Host("ERROR: Cannot update registry.")
    Exit(8);
}
Write-Host

if ( $domain_webstat_type -eq "awstats" ) {
    Write-Host("Cleaning AWStats database...")
    $db_files_ht = @{}
    $domain_webstat_db_files_location = $domain_webstat_path + "\awstats*.$domain_name.txt"
    $domain_webstat_db_files_list = dir $domain_webstat_db_files_location
    foreach ($file in $domain_webstat_db_files_list) {
        if ( $file -eq $null ) {
            continue;
        }
        $map_key = [datetime]::ParseExact([System.IO.Path]::GetFileNameWithoutExtension($file).Substring(7, 6), "MMddyy", $null)
        $db_files_ht.Add($map_key, $file)
    }
    $db_files_cl = $db_files_ht.GetEnumerator() | Sort-Object Key

    foreach ($item in $db_files_cl) {
        if ( $item -eq $null ) {
            continue;
        }

        $next_db_file_date = $item.key.AddMonths(1)

        if ( $from_date -ge $next_db_file_date ) {
            Write-Host("Skip: " + $item.value)
            continue;
        }

        if ( $from_date -lt $next_db_file_date -and $from_date -gt $item.key ) {
            Write-Host("Update: " + $item.value)

            $file_in = Get-Content($item.value)
            $file_out = @()
            foreach ($line in $file_in) {
                if ($line -like "LastLine *" -or $line -like "LastTime *" -or @($line) -like "LastUpdate *") {
                    continue;
                }

               $file_out += $line
            }

            ($file_out | Out-String) -replace "`r`n", "`n" | Out-File -filepath $item.value -force -Encoding ASCII
            continue;
        }

        if ( $from_date -le $item.key ) {
            Write-Host("Remove: " + $item.value)
            Remove-Item($item.value)
            continue;
        }
    }
    Write-Host

    $domain_webstat_dns_cache_file = $domain_webstat_path + "\dnscachelastupdate.domain.tld.txt"
    if ( Test-Path($domain_webstat_dns_cache_file) ) {
        Write-Host("Remove webstat DNS cache file...")
        Remove-Item($domain_webstat_dns_cache_file)
        Write-Host
    }
}

if ( $domain_webstat_type -eq "webalizer" ) {
    Write-Host("Cleaning Webalizer database...")
    $domain_webstat_current_file = $domain_webstat_path + "\webalizer.current"
    if ( Test-Path($domain_webstat_current_file) ) {
        Write-Host("Remove: " + $domain_webstat_current_file)
        Remove-Item($domain_webstat_current_file)
    }
    $domain_webstat_hist_file = $domain_webstat_path + "\webalizer.hist"
    if ( Test-Path($domain_webstat_hist_file) ) {
        Write-Host("Remove: " + $domain_webstat_hist_file)
        Remove-Item($domain_webstat_hist_file)
    }
    Write-Host
}

Write-Host("Cleaning vendor's and domains's traffic in system database...")
$from_date_sql_quoted = "'" + $from_date.ToString("yyyy-MM-dd") + "'"
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"SELECT date FROM clientstraffic WHERE cl_id=$domain_vendor_id and date>=$from_date_sql_quoted`"`"'
$cmd_out = Invoke-Expression $cmd
if ( $LastExitCode -ne 0 ) {
    Write-Host("ERROR: Cannot select vendor's traffic. Output:")
    Write-Host($cmd_out)
    Exit(9);
}

$cl_dates = [System.Collections.ArrayList]$cmd_out
$cl_dates.RemoveAt(0)   # remove header
foreach ($cl_date in $cl_dates) {
    if ($cl_date.Trim().length -eq 0) {
        continue
    }

    $date_quoted = "'" + $cl_date.Trim() + "'"

    Write-Host("Remove vendor's/domain's traffic for date: " + $date_quoted)

    $cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"DELETE FROM clientstraffic WHERE cl_id=$domain_vendor_id and date=$date_quoted`"`"'
    $cmd_out = Invoke-Expression $cmd
    if ( $LastExitCode -ne 0 ) {
        Write-Host("ERROR: Cannot delete vendor's traffic. Output:")
        Write-Host($cmd_out)
        Exit(10);
    }

    $cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"DELETE FROM domainstraffic WHERE dom_id=$domain_id and date=$date_quoted`"`"'
    $cmd_out = Invoke-Expression $cmd
    if ( $LastExitCode -ne 0 ) {
        Write-Host("ERROR: Cannot delete domain's traffic. Output:")
        Write-Host($cmd_out)
        Exit(11);
    }
}
Write-Host

Write-Host("Calculating statistics...")
$cmd = 'cmd /C `"`"%plesk_bin%\statistics.exe`" --calculate-one --domain-name=$domain_name`"'
$cmd_out = Invoke-Expression $cmd
if ( $LastExitCode -ne 0 ) {
    Write-Host("ERROR: Cannot calculate statistics. Output:")
    Write-Host($cmd_out)
    Exit(12);
}
Write-Host

Write-Host("Updating vendor's traffic in system database...")
# vendor's traffic update sql
###########################
# INSERT INTO ClientsTraffic 
# (cl_id, `date`, http_in, http_out, ftp_in, ftp_out, smtp_in, smtp_out, pop3_imap_in, pop3_imap_out) 
# SELECT d.vendor_id, dt.date, SUM(dt.http_in),	SUM(dt.http_out), 
#  SUM(dt.ftp_in), SUM(dt.ftp_out), SUM(dt.smtp_in), SUM(dt.smtp_out), 
#  SUM(dt.pop3_imap_in), SUM(dt.pop3_imap_out) 
# FROM (DomainsTraffic dt 
# INNER JOIN domains d ON dt.dom_id = d.id) 
# WHERE (d.vendor_id = $domain_vendor_id AND dt.date >= $from_date_sql_quoted) 
# GROUP BY d.vendor_id, dt.date
###########################
$cmd = 'cmd /C `"`"%plesk_bin%\dbclient.exe`" --direct-sql --sql=`"INSERT INTO ClientsTraffic `(cl_id`, ``date```, http_in`, http_out`, ftp_in`, ftp_out`, smtp_in`, smtp_out`, pop3_imap_in`, pop3_imap_out`) SELECT d.vendor_id`, dt.date`, SUM`(dt.http_in`)`, SUM`(dt.http_out`)`, SUM`(dt.ftp_in`)`, SUM`(dt.ftp_out`)`, SUM`(dt.smtp_in`)`, SUM`(dt.smtp_out`)`, SUM`(dt.pop3_imap_in`)`, SUM`(dt.pop3_imap_out`) FROM `(DomainsTraffic dt INNER JOIN domains d ON dt.dom_id = d.id`) WHERE `(d.vendor_id = $domain_vendor_id AND dt.date `>= $from_date_sql_quoted`) GROUP BY d.vendor_id`, dt.date`"`"'
$cmd_out = Invoke-Expression $cmd
if ( $LastExitCode -ne 0 ) {
    Write-Host("ERROR: Cannot update vendor's traffic. Output:")
    Write-Host($cmd_out)
    Exit(13);
}
Write-Host

Write-Host("Fixing log zips...")
$zip_location = $domain_logs_location + "\*.log_1.zip"
$zip_list = dir $zip_location
foreach ($file in $zip_list) {
    if ( $file -eq $null ) {
        continue;
    }

    Write-Host("Fix: " + $file)
    $to_file = $file.ToString().Replace('.log_1.zip', '.log.zip')

    Remove-Item($to_file)
    Rename-Item -path $file -newname $to_file
}
Write-Host

Write-Host("Calculating web statistics...")
$cmd = 'cmd /C `"`"%plesk_bin%\statistics.exe`" --generate-domain-webstat --domain-name=$domain_name`"'
$cmd_out = Invoke-Expression $cmd
if ( $LastExitCode -ne 0 ) {
    Write-Host("ERROR: Cannot calculate web statistics. Output:")
    Write-Host($cmd_out)
    Exit(14);
}

Write-Host
Write-Host("=================================================")
date
Write-Host

Exit(0);
